var Objects = require("convenience").Objects;

var HttpRequest = Packages.jodd.http.HttpRequest;
var HttpResponse = Packages.jodd.http.HttpResponse;

var MoreExecutors = com.google.common.util.concurrent.MoreExecutors;

var _ = require("lodash");

var DEFAULT_EXECUTOR = MoreExecutors.newDirectExecutorService();

function buildRequest (options)
{
    var request = HttpRequest.create(options.verb, options.uri);
    var key;

    for (key in options.headers)
    {
        request.header(key, options.headers[key]);
    }

    if (typeof options.data === "string")
    {
        request.body(options.data);
    }
    else if (typeof options.data === "object")
    {
        for (key in options.data)
        {
            request.form(key, options.data[key]);
        }
    }

    return request;
}

function execute (options)
{
    var request = buildRequest(options);
    var response = request.send();
    return {
        status: response.statusCode(),
        responseData: response.body(),
        responseHeaders: _(Objects.toJS(response.headerNames())).map(function (headerName) {
            return [headerName, response.header(headerName)];
        }).fromPairs().value()
    };
}

function HTTP (options)
{
    options = _({
        execute: execute,
        executor: DEFAULT_EXECUTOR
    }).extend(options).value();

    this.request = function request (uri)
    {
        return new Request({
            execute: options.execute,
            executor: options.executor,
            uri: uri
        });
    };
}

function Request (options)
{
    var data;
    var headers = {};
    var result;
    var verb = "get";

    var responseData;
    var responseHeaders;
    var status;

    function doExecute ()
    {
        var retval = (options.execute || execute)({
            uri: options.uri,
            data: data,
            headers: headers,
            verb: verb
        });
        responseData = retval.responseData;
        responseHeaders = retval.responseHeaders;
        status = retval.status;
    }

    this.data = function setData (value)
    {
        data = value;
        return this;
    };

    this.headers = function setHeaders (value)
    {
        headers = value;
        return this;
    };

    this.verb = function setVerb (value)
    {
        verb = value;
        return this;
    };

    this.execute = function execute ()
    {
        result = (options.executor || DEFAULT_EXECUTOR)["submit(java.util.concurrent.Callable)"](doExecute);
        return this;
    };

    this.asFuture = function asFuture ()
    {
        return result;
    };

    this.responseHeaders = function getResponseHeaders ()
    {
        return responseHeaders;
    };

    this.status = function getStatus ()
    {
        return status;
    };

    this.responseAsString = function getResponseAsString ()
    {
        return responseData;
    };
}

module.exports = HTTP;
