var _ = require("lodash");

/*
 * Get the IP address of the request.
 */
function getIPAddress (req)
{
    // https://tools.ietf.org/html/rfc7239#section-4
    if (req.hasHeader("Forwarded"))
    {
        var key = "for=";
        return _(req.header("Forwarded").split(",")).chain().find(function (entry) {
            return entry.toLowerCase().startsWith(key);
        }).map(function (entry) {
            entry = entry.substring(key.length()).split(";")[0];
            if (entry.startsWith('"') && entry.endsWith('"'))
            {
                entry = entry.substring(1, entry.length() - 1);
            }
            
            return entry;
        }).value() || req.remoteAddress().getHostString();
    }
    else if (req.hasHeader("X-Forwarded-For"))
    {
        return req.header("X-Forwarded-For").split(",")[0];
    }
    
    return req.remoteAddress().getHostString();
}

/*
 * Build a URL with base + path component. Ensures
 * that there is exactly one slash between them.
 */
function buildUrl (base, path)
{
    if (!base.endsWith("/"))
    {
        base = base + "/";
    }
    
    if (path.startsWith("/"))
    {
        path = path.substring(1);
    }
    
    return base + path;
}

module.exports = {
    getIPAddress: getIPAddress,
    buildUrl: buildUrl
};